﻿--[[
	Author: Darcey@AllForTheCode.co.uk
--]]

local origin = "BOTTOMLEFT";

-- # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
function initDCoordsMainFrame()
	log("initDCoordsMainFrame()");
	
	-- Addon gets obscured by other addons when using UIParent, see updateHandler for ALT+Z handling
	local uip = _G["UIParent"]
	local DCoordsMainFrameName = DCoords_MainFrameName .. tostring(math.random(99999999));
	log("DCoordsMainFrameName = " .. DCoordsMainFrameName);
	DCoordsMainFrame = CreateFrame("frame",DCoordsMainFrameName,uip);
	
	DCoordsMainFrame:RegisterEvent("ADDON_LOADED");
	DCoordsMainFrame:RegisterEvent("VARIABLES_LOADED");
	DCoordsMainFrame:RegisterEvent("ZONE_CHANGED_NEW_AREA");
	DCoordsMainFrame:RegisterEvent("ZONE_CHANGED_INDOORS");
	DCoordsMainFrame:RegisterEvent("ZONE_CHANGED");
	DCoordsMainFrame:RegisterEvent("PLAYER_LOGOUT");	
	
	DCoordsMainFrame:SetFrameStrata("HIGH")
	DCoordsMainFrame:SetWidth(DCoords_MainFrameWidth);
	DCoordsMainFrame:SetHeight(DCoords_MainFrameHeight);
	DCoordsMainFrame:EnableMouse(false);
	DCoordsMainFrame:SetResizable(true);
	DCoordsMainFrame:SetMovable(true);
	DCoordsMainFrame:SetPoint("BOTTOMLEFT",DCoords_MainFrameX,DCoords_MainFrameY);
	DCoordsMainFrame:SetScript("OnUpdate", function(self) DCoordsMainFrameUpdateHandler() end);
	DCoordsMainFrame:SetScript("OnMouseUp", function(self,buttonIdentifier) mouseUpHandler(self,buttonIdentifier) end);

	local DCoordsBackgroundType = DCoordsBgTypes[DCoordsSelectedBg];
	if (DCoordsBackgroundType == "classic") then
		DCoordsMainFrame:SetBackdrop({
			bgFile = "Interface/DialogFrame/UI-DialogBox-Background",
			edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
			tile = true, tileSize = 10, edgeSize = 10,
			insets = { left = 1, right = 1, top = 1, bottom = 1 }
		})		
	else
		DCoordsMainFrame.texture = DCoordsMainFrame:CreateTexture (nil, "background");
		DCoordsMainFrame.texture:SetColorTexture(DCoords_MainFrameBgR,DCoords_MainFrameBgG,DCoords_MainFrameBgB,DCoords_MainFrameBgA);
		DCoordsMainFrame.texture:SetAllPoints(true);
	end	
	
	DCoordsMainFrameText = DCoordsMainFrame:CreateFontString(nil, 'ARTWORK');
	DCoordsMainFrameText:SetAllPoints(true);
	DCoordsMainFrameText:SetFont(getMainFrameFont(), DCoords_MainFrameFontSize,"OUTLINE");
	DCoordsMainFrameText:SetShadowOffset(1, -1);
	DCoordsMainFrameText:SetTextColor(DCoords_MainFrameTextR, DCoords_MainFrameTextG, DCoords_MainFrameTextB,DCoords_MainFrameTextA);
	DCoordsMainFrameText:SetNonSpaceWrap(false);
	DCoordsMainFrameText:SetText("Darcey@AllForTheCode.co.uk");
	--DCoordsMainFrameText:SetJustifyH("LEFT");
	DCoordsMainFrameText:SetJustifyH(alignmentTypes[DCoords_MainFrameTextAlign]);
	
	
	
	--[[
	DCoordsMainFrame:EnableMouseWheel (true);
	DCoordsMainFrame:SetScript("OnMouseWheel", function (self, delta)
		if (delta > 0) then
			DCoords_MainFrameWidth = DCoords_MainFrameWidth+10;
			DCoords_MainFrameHeight = DCoords_MainFrameHeight+0.4;
			DCoords_MainFrameFontSize = DCoords_MainFrameFontSize+0.3;
		elseif (delta < 0) then
			if (DCoords_MainFrameWidth < 100) then
				return;
			end
			DCoords_MainFrameWidth = DCoords_MainFrameWidth-10;
			DCoords_MainFrameHeight = DCoords_MainFrameHeight-0.4;
			DCoords_MainFrameFontSize = DCoords_MainFrameFontSize-0.3;
		end
		self:SetWidth(DCoords_MainFrameWidth);
		self:SetHeight(DCoords_MainFrameHeight);
		DCoordsMainFrameText:SetFont(getMainFrameFont(), DCoords_MainFrameFontSize)
		DCoordsMainFrameText:SetWidth(DCoords_MainFrameWidth);
		DCoordsMainFrameText:SetHeight(20);
		
	end)
	--]]
	
	
	
	DCoordsMainFrame:SetScript("OnEvent", function(self,event,addonName)
		if (event == "ADDON_LOADED") then
			if (addonName == "DCoords") then
				log("DCoords v"..DCoordsVersion.." ADDON_LOADED!");
			end
		elseif (event == "VARIABLES_LOADED") then
			log("DCoords v"..DCoordsVersion.." VARIABLES_LOADED!");
			DCoordsInit();
		else
			log("DCoords: WARNING: Unhandled event: [" .. event .. "]");
		end
	end);
	
end
-- # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #




-- # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
function updateDCoordsMainFrame()
	log("updateDCoordsMainFrame()");
	-- Width, Height and Font
	DCoordsMainFrame:SetWidth(DCoords_MainFrameWidth);
	DCoordsMainFrame:SetHeight(DCoords_MainFrameHeight);
	DCoordsMainFrameText:SetFont(getMainFrameFont(), DCoords_MainFrameFontSize,"OUTLINE");
	
	-- Position (Had to do it manually)
	--DCoordsMainFrame:SetPoint(origin,DCoords_MainFrameX,(DCoords_MainFrameY-10));
	DCoordsMainFrame:SetPoint(origin,DCoords_MainFrameX,DCoords_MainFrameY);

	-- Drag
	if (DCoords_MainFrameDragEnabled == true) then
		
		DCoordsMainFrame:RegisterForDrag("LeftButton");
		
		DCoordsMainFrame:SetScript("OnDragStart", function(self) 
			toChat("DCoords: Drag started!"); self:StartMoving(); 
		end);
		
		DCoordsMainFrame:SetScript("OnDragStop", function(self)
			DCoords_MainFrameX = DCoordsMainFrame:GetLeft();
			DCoords_MainFrameY = DCoordsMainFrame:GetTop()-DCoordsMainFrame:GetHeight();
			self:StopMovingOrSizing()
		end);
			
	else
		DCoordsMainFrame:RegisterForDrag(nil);
		DCoordsMainFrame:SetScript("OnDragStart",nil);
		DCoordsMainFrame:SetScript("OnDragStop", nil);
	end
	

	-- BG
	local tileSize = 1;
	local edgeSize = 16;
	
	if (DCoords_MainFrameShowBg == true) then

		local DCoordsBackgroundType = DCoordsBgTypes[DCoords_MainFrameSelectedBg];
		if (DCoordsBackgroundType == "classic") then
			DCoordsMainFrame.texture:SetColorTexture(0,0,0,0);
			DCoordsMainFrame:SetBackdrop({
				bgFile = "Interface/DialogFrame/UI-DialogBox-Background",
				edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
				tile = true, tileSize = tileSize, edgeSize = edgeSize,
				insets = { left = 5, right = 5, top = 5, bottom = 5 }
			})
		else
			DCoordsMainFrame:SetBackdrop(nil);
			DCoordsMainFrame.texture:SetColorTexture(DCoords_MainFrameBgR,DCoords_MainFrameBgG,DCoords_MainFrameBgB,DCoords_MainFrameBgA);
		end	
	
	else
		DCoordsMainFrame.texture:SetColorTexture(0,0,0,0);
		DCoordsMainFrame:SetBackdrop(nil);
	end
	
	

	-- FontSize (NOTE: Height is only usable for small sizes, use SetTExtHeight to fontsize to fix)
	local txtW = DCoordsMainFrameText:GetStringWidth();
	--local txtH = DCoordsMainFrameText:GetStringHeight();
	DCoordsMainFrame:SetWidth(txtW);
	DCoordsMainFrame:SetHeight(DCoords_MainFrameFontSize);
	--DCoordsMainFrameText:SetTextHeight(DCoords_MainFrameFontSize); 
	
	
	-- Text color	
	DCoordsMainFrameText:SetTextColor(DCoords_MainFrameTextR,DCoords_MainFrameTextG,DCoords_MainFrameTextB,DCoords_MainFrameTextAlpha);
	
	-- TExt alpha
	DCoordsMainFrameText:SetAlpha(DCoords_MainFrameTextAlpha);
	
	
	-- Visibility
	if (DCoords_MainFrameVisible == true) then
		DCoordsMainFrame:Show();
	else
		DCoordsMainFrame:Hide();
	end

	
end
-- # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #










-- # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
function DCoordsSlashCommandHandler(cmd)
	log("DCoordsSlashCommandHandler(): " .. cmd);
	
	if (cmd == nil or cmd == "" or string.find(cmd, "help") ~= nil) then
		toChat(" ");
		toChat("DCoords v" .. DCoordsVersion .. " slash commands:");
		toChat("  /DC show menu - show menu");
		toChat("  /DC hide menu - hide menu");
		toChat("  /DC menu - toggle menu visibility");
		toChat("  /DC show  - show coordinates");
		toChat("  /DC hide - hide coordinates");
		toChat("  /DC reset - reset position and settings");
		return;
	end
	
	strSearchResult = string.find(cmd,"reset");
	if (strSearchResult ~= nil) then
		toChat("DCoords attempting to reset positions...");
		reset();
		return;
	end
	
	if (cmd == "menu") then
		if (DCoordsMenuFrame:IsVisible()) then
			DCoordsMenuFrame:Hide();
		else
			DCoordsMenuFrame:Show();
		end
		return;
	end
	
	if (cmd == "show menu") then
		DCoordsMenuFrame:Show();
		return;
	end
	
	if (cmd == "hide menu") then
		DCoordsMenuFrame:Hide();
		return;
	end
	
	if (cmd == "show") then
		DCoordsMainFrame:Show();
		return;
	end
	
	if (cmd == "hide") then
		DCoordsMainFrame:Hide();
		return;
	end
	
	
	updateDCoordsMainFrame();
	updateDCoordsMapFrameState();
	updateDCoordsMenuFrameState();

	
end
-- # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #





-- eventHandlerFunctions
-- # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
-- # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

function mouseUpHandler(self,buttonIdentifier)

	if (buttonIdentifier == "RightButton") then
		playClick();
		if (DCoordsMenuFrame:IsVisible()) then
			DCoordsMenuFrame:Hide();
		else
			DCoordsMenuFrame:Show();
		end
	end
end
-- # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #







-- Global functions
-- # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
local DCoordsMainFrameTextOldValue = "";
-- # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
setDCoordsMainFrameText = function(value)
	--DCoordsMainFrameText:SetJustifyH("LEFT");
	DCoordsMainFrameText:SetJustifyH(alignmentTypes[DCoords_MainFrameTextAlign]);
	DCoordsMainFrameText:SetFont(getMainFrameFont(), DCoords_MainFrameFontSize,"OUTLINE");
	if (value == nil or value == "") then
		--log("setDCoordsMainFrameText(): Nil handler");
		DCoordsMainFrameText:SetText( DCoordsMainFrameText:GetText() );
	else
		if (DCoordsMainFrameTextOldValue == value) then
			--log("setDCoordsMainFrameText(): oldValue == newValue = no update");
		else
			--log("setDCoordsMainFrameText(): Value updated");
			DCoordsMainFrameText:SetText(value);
			DCoordsMainFrameTextOldValue = value;
		end
	end
	
	-- SetJustifyH sometimes works and then sometimes doesn't best not to use it!
	DCoordsMainFrameText:SetJustifyH(alignmentTypes[DCoords_MainFrameTextAlign]);
	--DCoordsMainFrameText:SetJustifyH("LEFT");  
	--log(DCoordsMainFrameText:GetJustifyH());
end
-- # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #



function DCoordsInit()
	log("DCoordsInit()");

	-- One time calls
	SetMapToCurrentZone(); -- Works anyway but could be another addon doing that
		
	-- DO NOT CALL DCoordsMainFrame() as it's called at the base of this file
	
	-- DCoordsMap
	initDCoordsMapFrame();
	
	-- DCoordsMenu
	initDCoordsMenuFrame();
	
	-- Update for full init
	updateDCoordsMainFrame();
	updateDCoordsMapFrameState();
	updateDCoordsMenuFrameState();


	
	-- Setup slash commands
	SlashCmdList["DCoordsCOMMAND"] = DCoordsSlashCommandHandler;
	SLASH_DCoordsCOMMAND1 = "/dc";
	
	toChat(" ");
	toChat("DCoords v"..DCoordsVersion);
	toChat("DCoords: To view the menu right click coordinates bar or");
	toChat("DCoords: type /DC menu.");
	toChat("DCoords: For more information type /DC help or /DC");
	toChat(" ");
end
-- # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #











-- init
-- # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
initDCoordsMainFrame();
-- # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #










